import os
import sys
sys.path.append(os.getcwd())
from ranking_system import *
from searchdatamodels import *
import unittest
    

class RankingTest(unittest.TestCase):
    def setUp(self):
        self.location_query="who was in paris or rome"
    def test_rank_candidates(self):
        correct_name="Indiana Jones"
        candidate_list=[
            Candidate(Name="Rick Deckard", WorkExperienceList=[WorkExperience(Specialization="policeman", Institution="Los Angeles Police Department")] ),
            Candidate(Name=correct_name, WorkExperienceList=[WorkExperience(Specialization="history professor", Institution="University of Chicago")]),
            Candidate(Name="Han Solo", WorkExperienceList=[WorkExperience(Specialization="Smuggler", Institution="Self-Employed")])
        ]
        ranked_candidates= rank_candidates("professor", candidates=candidate_list)
        self.assertEqual(ranked_candidates[0].Name, correct_name.lower())

    def test_expand_locations_none(self):
        query_locations=["switzerland and canada"]
        expanded_locations=get_expanded_locations_geopy(query_locations)
        self.assertEqual(0, len(expanded_locations))

    def test_get_expanded_locations_geopy(self):
        query_locations=["zurich, switzerland"]
        expanded_locations=get_expanded_locations_geopy(query_locations)
        self.assertEqual(1, len(expanded_locations))

    def test_get_expanded_locations_llm(self):
        query_locations=["zurich"]
        expanded_locations=get_expanded_locations_llm(query_locations)
        self.assertTrue("Basel" in expanded_locations)

    def test_get_expanded_locations_llm_state(self):
        query_locations=["zurich"]
        expanded_locations=get_expanded_locations_llm(query_locations, include_state=True)
        self.assertTrue("Basel, Switzerland" in expanded_locations)

    def test_calculate_skill_match_score_same(self):
        skills=["dance"]
        score=calculate_skill_match_score(skills,skills)
        self.assertEqual(1, score)

    def test_calculate_skill_match_score(self):
        extracted_skills=["teaching", "writing"]
        good_candidate_skills=["education", "literature"]
        bad_candidate_skills=["marksmanship", "hunting"]
        good_score=calculate_skill_match_score(extracted_skills, good_candidate_skills)
        bad_score=calculate_skill_match_score(extracted_skills, bad_candidate_skills)
        self.assertGreater(good_score, bad_score)

    def test_calculate_location_match_score_same(self):
        locations=["madrid"]
        candidate_location='madrid'
        score=calculate_location_match_score(locations, locations, candidate_location)
        self.assertEqual(1, score)

    def test_calculate_location_match_score(self):
        query_location=["not real location"]
        good_locations=["amsterdam"]
        bad_locations=["berlin"]
        candidate_location='brussels'
        good_score=calculate_location_match_score(query_location, good_locations, candidate_location)
        bad_score=calculate_location_match_score(query_location, bad_locations, candidate_location)
        self.assertGreater(good_score, bad_score)

    def test_calculate_similarity_score(self):
        good_candidate=Candidate(Name="Jay Carter", WorkExperienceList=[WorkExperience(Specialization="Rapper")])
        bad_candidate=Candidate(Name="Jimmy Carter", WorkExperienceList=[WorkExperience(Specialization="President")])
        query="musician"
        good_score=calculate_similarity_score(query, good_candidate)
        bad_score=calculate_similarity_score(query, bad_candidate)
        self.assertGreater(good_score, bad_score)

    def test_calculate_total_score_no_location_no_skills(self):
        good_candidate=Candidate(Name="Jay Carter", WorkExperienceList=[WorkExperience(Specialization="Rapper")])
        bad_candidate=Candidate(Name="Jimmy Carter", WorkExperienceList=[WorkExperience(Specialization="President")])
        query="musician"
        good_score=calculate_total_score(query, good_candidate)
        bad_score=calculate_total_score(query, bad_candidate)
        self.assertGreater(good_score, bad_score)

    def test_calculate_total_score_no_location(self):
        good_candidate=Candidate(Name="Jay Carter", WorkExperienceList=[WorkExperience(Specialization="Rapper")], Skills=["singing", "dancing"])
        bad_candidate=Candidate(Name="Jimmy Carter", WorkExperienceList=[WorkExperience(Specialization="President")], Skills=["speaking"])
        query="musician skilled in singing"
        good_score=calculate_total_score(query, good_candidate)
        bad_score=calculate_total_score(query, bad_candidate)
        self.assertGreater(good_score, bad_score)

    def test_calculate_total_score_no_skills(self):
        good_candidate=Candidate(Name="Jay Carter", WorkExperienceList=[WorkExperience(Specialization="Rapper")], Location="New York City")
        bad_candidate=Candidate(Name="Jimmy Carter", WorkExperienceList=[WorkExperience(Specialization="President")], Location="Washington DC")
        query="musician located in manhattan"
        good_score=calculate_total_score(query, good_candidate)
        bad_score=calculate_total_score(query, bad_candidate)
        self.assertGreater(good_score, bad_score)

    def test_calculate_total_score(self):
        good_candidate=Candidate(Name="Jay Carter", WorkExperienceList=[WorkExperience(Specialization="Rapper")], Location="New York City",Skills=["singing", "dancing"])
        bad_candidate=Candidate(Name="Jimmy Carter", WorkExperienceList=[WorkExperience(Specialization="President")], Location="Washington DC",Skills=["speaking"])
        query="musician located in manhattan skilled in singing"
        good_score=calculate_total_score(query, good_candidate)
        bad_score=calculate_total_score(query, bad_candidate)
        self.assertGreater(good_score, bad_score)

    def test_get_top_candidates(self):
        correct_name="Indiana Jones"
        candidate_list=[
            Candidate(Name="Rick Deckard", WorkExperienceList=[WorkExperience(Specialization="policeman", Institution="Los Angeles Police Department")] ),
            Candidate(Name=correct_name, WorkExperienceList=[WorkExperience(Specialization="history professor", Institution="University of Chicago")]),
            Candidate(Name="Han Solo", WorkExperienceList=[WorkExperience(Specialization="Smuggler", Institution="Self-Employed")])
        ]
        ranked_candidates= get_top_candidates("professor", candidates=candidate_list,k=3)
        self.assertEqual(ranked_candidates[0].Name, correct_name.lower())


if __name__ =='__main__':
    unittest.main()


import os
import sys
sys.path.append(os.getcwd())
from extraction_utils import *
import unittest
from unittest.mock import patch
from munch import DefaultMunch

class OpenAIResponseMock:
    def __init__(self,text):
        self.choices=[DefaultMunch.fromDict({"text":text})]

class ExtractingTest(unittest.TestCase):
    def setUp(self):
        self.location_query="who was in paris or rome"
        self.bad_json_str="bad json string} ^[jkksd]\'__ 0"
        self.bad_openai_response=OpenAIResponseMock(self.bad_json_str)

    def test_extract_location_mentions_spacy(self):
        locations=extract_location_mentions_spacy(query=self.location_query)
        self.assertTrue("paris" in locations)
        self.assertTrue("rome" in locations)

    def test_extract_location_mentions_llm(self):
        locations=extract_location_mentions_llm(self.location_query)
        self.assertTrue("paris" in locations)
        self.assertTrue("rome" in locations)

    def test_extract_skills(self):
        query="find me someone proficient in metallurgy and welding"
        skills=extract_skills(query)
        self.assertTrue("metallurgy" in skills)
        self.assertTrue("welding" in skills)

    def test_extract_education_empty(self):
        query="hello"
        education=extract_education(query=query)
        self.assertEqual(0, len(education))

    def test_extract_education(self):
        degree_0="phd"
        degree_1="masters"
        major_0="chemistry"
        major_1="neuroscience"
        school_0="oxford university"
        school_1="cambridge university"
        target_list=[degree_0, degree_1, major_0, major_1, school_0, school_1]
        for x in range(len(target_list)):
            text="find me someone"
            if x>=0:
                text+=f" with a {degree_0}"
            if x>=1:
                text+=f" or a {degree_1}"
            if x>=2:
                text+=f" in {major_0}"
            if x>=3:
                text+=f" or {major_1}"
            if x>=4:
                text+=f" who attended {school_0}"
            if x>=5:
                text+=f" or {school_1}"
            with self.subTest(text=text):
                education_dict=extract_education(query=text)
                if x>=0:
                    self.assertIn("Degree", education_dict)
                    self.assertIn(degree_0, education_dict["Degree"])
                if x>=1:
                    self.assertIn(degree_1, education_dict["Degree"])
                if x>=2:
                    self.assertIn("Specialization", education_dict)
                    self.assertIn(major_0, education_dict["Specialization"])
                if x>=3:
                    self.assertIn(major_1, education_dict["Specialization"])
                if x>=4:
                    self.assertIn("Institution", education_dict)
                    self.assertIn(school_0, education_dict["Institution"])
                if x>=5:
                    self.assertIn(school_1, education_dict["Institution"])

    def test_extract_employment_empty(self):
        query="good afternoon"
        employment_dict=extract_employment(query)
        self.assertEqual(0, len(employment_dict))

    def test_extract_employment(self):
        title_0="engineer"
        title_1="developer"
        company_0="google"
        company_1="microsoft"
        target_list=[title_0, title_1, company_0, company_1]
        for x in range(len(target_list)):
            query="find me a person "
            if x>=0:
                query+=f" who worked as an {title_0}"
            if x>=1:
                query+=f" or a {title_1}"
            if x>=2:
                query+=f" at {company_0}"
            if x>=3:
                query+=f" or {company_1}"
            with self.subTest(query=query):
                employment_dict=extract_employment(query)
                if x>=0:
                    self.assertIn("Specialization", employment_dict)
                    self.assertIn( title_0,employment_dict["Specialization"])
                if x>=1:
                    self.assertIn(title_1,employment_dict["Specialization"])
                if x>=2:
                    self.assertIn("Institution", employment_dict)
                    self.assertIn(company_0,employment_dict["Institution"])
                if x>=3:
                    self.assertIn(company_1,employment_dict["Institution"])

    @patch("extraction_utils.openai.Completion.create")
    def test_extract_error(self,openai_Completion_create_mock):
        openai_Completion_create_mock.return_value=self.bad_openai_response
        for function in [extract_education, extract_employment]:
            with self.subTest(function=function):
                try:
                    function("any query")
                except Exception as exception:
                    self.assertIsInstance(exception, json.decoder.JSONDecodeError)
                    print(exception.msg)
                    self.assertNotEqual(-1, exception.msg.find(self.bad_json_str))




if __name__=='__main__':
    unittest.main()


import os
import sys
sys.path.append(os.getcwd())
from talentiumkg import KnowledgeGraph
import openai
from searchdatamodels import SearchTemplate
from typing import List
try:
    # Trying to find module in the parent package
    from .extraction_utils import *
    from .ranking_system import *
except ImportError:
    print('Relative import failed')

try:
    # Trying to find module on sys.path
    from extraction_utils import *
    from ranking_system import *
except ModuleNotFoundError:
    print('Absolute import failed')

if "OPENAI_API_KEY" in os.environ:
    openai.api_key = os.environ["OPENAI_API_KEY"]
    #for local testing, add the openai api_key to your env
    #otherwise make sure that you set your openai api_key before you use this library
INSTITUTION='Institution'
SPECIALIZATION="Specialization"
knowledge_graph=KnowledgeGraph()

# generate a set of queries similar to the original
def generate_expanded_queries(query: str, num_queries: int = 5) -> List[str]:
    '''The `generate_expanded_queries` function takes a user's search query and generates a list of similar
    queries that are relevant to the original context, such as similar job titles, nearby locations, and
    similar skills.
    
    Parameters
    ----------
    query : str
        The `query` parameter is a string that represents the user's search query. It is the original query
    for which we want to generate similar queries.
    num_queries : int, optional
        The `num_queries` parameter specifies the number of similar queries that should be generated. By
    default, it is set to 5, but you can change it to any positive integer value.
    
    Returns
    -------
        The function `generate_expanded_queries` returns a list of similar queries that are relevant to the
    original query context. The list includes `num_queries` similar queries, as well as the original
    query itself.
    
    '''
    prompt = (
        f"Given a user's search query, generate a list of {num_queries} similar queries that are relevant to the original context [similar job title, nearby location, similar skills].\n\n"
        "Examples:\n"
        "1. Input Query: \"Software engineer in New York\"\n"
        "   [\"Software Development Engineer in New York\", \"Software developer in New York\", \"SDE in Jersey City\" ]\n\n"
        "2. Input Query: \"Remote Data Scientist\"\n"
        "   [\"Remote Data Science Engineer\", \"Work from home Data Scientist\"]\n\n"
        "3. Input Query: \"Entry-level software developer\"\n"
        "   [\"Software engineering intern\", \"Junior software engineer\"]\n\n"
        "4. Input Query: \"Experienced software engineer\"\n"
        "   [\"Senior software engineer\", \"Software Architect\"]\n\n"
        "5. Input Query: \"Software development careers\"\n"
        "   [\"Software engineering job paths\", \"Career options in software development\"]\n\n"
        f"Input Query: \"{query}\"\n"
    )

    
    response = openai.Completion.create(
        engine="text-davinci-003",  
        prompt=prompt,
        max_tokens=200,
        stop=None,
        temperature=0,
        n=num_queries,
        frequency_penalty=0.5,
        presence_penalty=0.5
    )

    expanded_text = response.choices[0]['text']
    print(expanded_text)

    # Find the start index of the list
    result_list_start = expanded_text.index("[")  

    # Find the end index of the list
    result_list_end = expanded_text.index("]")  

    # extract the result
    result_list_str = expanded_text[result_list_start:result_list_end+1] 
    result_list = eval(result_list_str) 

    # add original query as well
    result_list.append(query)

    return result_list


def get_search_template_from_query(user_query:str) -> SearchTemplate:
    '''The function extracts information from a user query to create a search template for employment and
    education.
    
    Parameters
    ----------
    user_query : str
        The user_query parameter is a string that represents the user's search query. It is the input
    provided by the user when they are searching for something.
    

    Returns
    -------
    SearchTemplate
        an instance of the SearchTemplate class properly formatted
    '''
    employment_dict=extract_employment(user_query)
    title_list=[]
    company_list=[]
    if INSTITUTION in employment_dict:
        company_list=employment_dict[INSTITUTION]
    if SPECIALIZATION in employment_dict:
        title_list=employment_dict[SPECIALIZATION]
    
    education_dict=extract_education(user_query)
    school_list=[]
    major_list=[]
    if INSTITUTION in education_dict:
        school_list=education_dict[INSTITUTION]
    if SPECIALIZATION in education_dict:
        major_list=education_dict[SPECIALIZATION]
    location_list=extract_location_mentions_llm(user_query)
    skill_list=extract_skills(user_query)

    return SearchTemplate(location=location_list, 
                          skill=skill_list, title=title_list, 
                          company=company_list, 
                          school=school_list, 
                          major=major_list)



def expand_search_template_with_kg(search_template: SearchTemplate) -> SearchTemplate:
    knowledge_graph=KnowledgeGraph()
    new_skill_set=set(search_template.skill)
    for single_skill in search_template.skill:
        new_skill_set.update(knowledge_graph.infer_similar_skill(single_skill))
    new_title_set=set(search_template.title)
    for single_title in search_template.title:
        new_title_set.update(knowledge_graph.infer_similar_job_title(single_title))
    search_template.skill=[s for s in new_skill_set]
    search_template.title=[t for t in new_title_set]
    return search_template


def generate_mongo_query_from_template_and_embedding(embedding: list[float],k:int,template: SearchTemplate=None)->dict:
    '''The function generates a MongoDB query based on a given template and embedding.
    
    Parameters
    ----------
    embedding : list[float]
        The `embedding` parameter is a list of floats representing a vector embedding of a document or
    query. It is used to find similar documents based on their embeddings.
    template : SearchTemplate
        The `template` parameter is an instance of the `SearchTemplate` class. It contains the search
    criteria for the query, such as titles, locations, skills, and majors.
    
    Returns
    -------
        a dictionary that represents a MongoDB query.
    
    '''
    if template == None or (template.title==[] and template.location==[] and template.skill==[] and template.major==[]):
        res = {"$search":{
        "index": "default",
        "knnBeta": {
        "vector": embedding,
        "path": "Embedding",
        "k": k,
        }}}
    else:
        can_filter = {"compound":{}}
        can_filter["compound"]["must"] = []
        if len(template.title) > 0:
            can_filter["compound"]["must"].append({"compound": {"should": []}})
        for title in template.title:
            can_filter['compound']['must'][-1]['compound']['should'].append({
                  "phrase": {
                    "path": ["WorkExperienceList.Specialization", "Summary.Text"],
                    "query": title,
                  }
                })
        if len(template.location) > 0:
            can_filter["compound"]["must"].append({"compound": {"should": []}})
        for location in template.location:
            can_filter['compound']['must'][-1]['compound']['should'].append({
                  "phrase": {
                    "path": "Location",
                    "query": location,
                  }
                })
        if len(template.skill) > 0:
            can_filter["compound"]["must"].append({"compound": {"should": []}})
        for skill in template.skill:
            can_filter['compound']['must'][-1]['compound']['should'].append({
                  "phrase": {
                    "path": ["Skills", "Summary.Text"],
                    "query": skill,
                  }
                })
        if len(template.major) > 0:
            can_filter["compound"]["must"].append({"compound": {"should": []}})
        for major in template.major:
            can_filter['compound']['must'][-1]['compound']['should'].append({
                  "phrase": {
                    "path": ["EducationExperienceList.Specialization", "Summary.Text"],
                    "query": major,
                  }
                })
        res = {"$search":{
        "index": "default",
        "knnBeta": {
        "vector": embedding,
        "path": "Embedding",
        "k": k,
        "filter": can_filter
          }}}
    return res